(*********************************************************************)
(*                         Diy                                       *)
(*                                                                   *)
(*   Jade Alglave, Luc Maranget INRIA Paris-Rocquencourt, France.    *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique. All rights reserved. This file is distributed    *)
(*  under the terms of the Lesser GNU General Public License.        *)
(*********************************************************************)

module type I = sig
 type arch_reg

 val pp_reg : arch_reg -> string
 val free_registers : arch_reg list
end

module type S = sig
  type arch_reg

(* Locations *)
  type location =
    | Reg of Code.proc * arch_reg
    | Loc of Code.loc

  val pp_location : location -> string

(* Initial & final states *)
  type init = (location * string) list
  type final = (location * int) list

(***********************)
(* Register allocation *)
(***********************)

  type st
  val st0 : st

  val alloc_reg : st -> arch_reg * st

end

module Make(I:I) : S with type arch_reg = I.arch_reg
= struct
  type arch_reg = I.arch_reg

  type location =
    | Reg of int * arch_reg
    | Loc of string

  type init = (location * string) list
  type final = (location * int) list

  let pp_location = function
    | Reg (i,r) -> Printf.sprintf "%i:%s" i (I.pp_reg r)
    | Loc loc -> loc

  type st = arch_reg list

  let st0 = I.free_registers

  let alloc_reg = function
    | [] -> Warn.fatal "No more registers"
    | r::rs -> r,rs
end
