(*********************************************************************)
(*                         Diy                                       *)
(*                                                                   *)
(*   Jade Alglave, Luc Maranget INRIA Paris-Rocquencourt, France.    *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique. All rights reserved. This file is distributed    *)
(*  under the terms of the Lesser GNU General Public License.        *)
(*********************************************************************)

open Printf

exception BadOpt of string * string

let bad_opt opt correct = raise (BadOpt (opt,correct))

type spec = string * Arg.spec * string


let arch_opt f default =
  ["-arch",
   Arg.String
     (fun s ->
       let a = match Misc.parse_arch s with
       | Some a -> a
       | None -> bad_opt "-arch" "X86/PPC" in
       f a),
     sprintf "<X86|PPC> set architecture (default %s)" default]
       

let parse_cmdline options get_cmd_arg =
  try begin
    Arg.parse options
      get_cmd_arg
      (sprintf "Usage %s [options] [arg]*
  options are:" Sys.argv.(0))
  end with BadOpt (opt,correct) ->
    eprintf "'%s' argument must be one of %s\n" opt correct ;
    exit 2
