(*********************************************************************)
(*                          Litmus                                   *)
(*                                                                   *)
(*        Luc Maranget, INRIA Paris-Rocquencourt, France.            *)
(*        Susmit Sarkar, University of Cambridge, UK.                *)
(*                                                                   *)
(*  Copyright 2010 Institut National de Recherche en Informatique et *)
(*  en Automatique and the authors. All rights reserved.             *)
(*  This file is distributed  under the terms of the Lesser GNU      *)
(*  General Public License.                                          *)
(*********************************************************************)

module type S = sig
  module A : Arch.S
  module C : Constr.S with module A = A

  type run_type = Int | Pointer

  type 'a type_env = ('a * run_type) list
  type t =
    { init : A.state ;
      info : MiscParser.info ;
      code : (int * (A.Out.t * A.reg type_env)) list ;
      condition : C.constr ;
      globals : string type_env ;
      flocs : A.location list ; }

  val find_our_constraint :
      C.constr -> (string * ConstrGen.kind) list -> C.constr
  val get_nprocs : t -> int
end

    

module Make(A:Arch.S) : S with module A = A =
struct
  module A  = A
  module C = Constr.Make(A)

  type run_type = Int | Pointer
  type 'a type_env = ('a * run_type) list
  type t =
    { init : A.state ;
      info : MiscParser.info ;
      code : (int * (A.Out.t * A.reg type_env)) list ;
      condition : C.constr ;
      globals : string type_env ;
      flocs : A.location list ; }

	(* Code Duplication, argh! Where is the right place to put this? 
	   (As it stands, this is a copy, by hand, of code in mem.new/semExtra.ml) *)
    let find_our_constraint bare quantifiers = 
      let quantifier = 
      try
	List.assoc (Globals.get_runname ()) quantifiers 
      with
	Not_found -> 	
	  try 
	    List.assoc "default" quantifiers 
	  with 
	    Not_found -> (* None Specified *) ConstrGen.Unconstrained 
      in 
      match bare,quantifier with
      | ConstrGen.QueryState p,ConstrGen.Require -> ConstrGen.ForallStates p
      | ConstrGen.QueryState p,ConstrGen.Allow -> ConstrGen.ExistsState p
      | ConstrGen.QueryState p,ConstrGen.Forbid -> ConstrGen.NotExistsState p
      | _,_ -> bare

  let get_nprocs t = List.length t.code
end
